##
# PloneSVNaccess<br>
#  Providing read and write access to subversion.<br>
#<br>
# PloneSVNaccessWorkflow.py<br>
#  Providing the PloneSVNaccess Workflow (Different Permissions).<br>
#<br>
# Author: Christian Koller 'admin@iecw.net'.<br>
#<br>
# Released under GNU GPL v2 or later.
##

from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.PloneSVNaccess.config import PROJECTNAME, GLOBALS, add_permission, edit_permission, view_permission, write_permission, webdav_permission, view_role, write_role, webdav_role

##
# PloneSVNaccess Workflow<br>
# Providing the PloneSVNaccessWorkflow with different permissions (view_permission, webdav_permission, write_permission),<br>
# different states, and the neccesary transitions ('Read and write restricted Transition', 'Read to authenticated,
# write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 
# 'Read to all write to authenticated Transition', 'Read and write to all Transition')
def svnWf(id):
	""" PloneSVNaccess Workflow
		Providing the PloneSVNaccessWorkflow with different permissions (view_permission, webdav_permission, write_permission),
		different states, and the neccesary transitions ('Read and write restricted Transition', 'Read to authenticated,
		write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 
		'Read to all write to authenticated Transition', 'Read and write to all Transition')
	"""
	workflowDef = DCWorkflowDefinition(id)
	
	workflowDef.setProperties(title='PloneSVNaccess Workflow')
	
	workflowDef.addManagedPermission(view_permission)
	workflowDef.addManagedPermission(webdav_permission)
	workflowDef.addManagedPermission(write_permission)

	for wfTransi in ['Read and write restricted Transition', 'Read to authenticated, write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 'Read to all write to authenticated Transition', 'Read and write to all Transition']:
		workflowDef.transitions.addTransition(wfTransi)

	for wfVariables in ['action', 'time', 'comments', 'actor', 'review_history']:
		workflowDef.variables.addVariable(wfVariables)
	
	
	workflowDef.states.addState('Read and write restricted')
	wfStrwrest = workflowDef.states['Read and write restricted']
	wfStrwrest.setProperties(title="""Read and write restricted""",
				transitions=('Read to authenticated, write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 'Read to all write to authenticated Transition', 'Read and write to all Transition'))
	wfStrwrest.setPermission(add_permission, 0, ['Manager', 'Owner'])
	wfStrwrest.setPermission(edit_permission, 0, ['Manager', 'Owner'])
	wfStrwrest.setPermission(view_permission, 0, [webdav_role, write_role, view_role, 'Manager', 'Owner'])
	wfStrwrest.setPermission(webdav_permission, 0, [webdav_role, 'Manager', 'Owner'])
	wfStrwrest.setPermission(write_permission,0, [write_role, 'Manager', 'Owner'])

	workflowDef.states.addState('Read to authenticated, write restricted')
	wfStrauwrest = workflowDef.states['Read to authenticated, write restricted']
	wfStrauwrest.setProperties(title="""Read to authenticated, write restricted""",
				transitions=('Read and write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 'Read to all write to authenticated Transition', 'Read and write to all Transition'))
	wfStrauwrest.setPermission(add_permission, 0, ['Manager', 'Owner'])
	wfStrauwrest.setPermission(edit_permission, 0, ['Manager', 'Owner'])
	wfStrauwrest.setPermission(view_permission, 0, ['Authenticated', 'Manager', 'Owner'])
	wfStrauwrest.setPermission(webdav_permission, 0, [webdav_role, 'Manager', 'Owner'])
	wfStrauwrest.setPermission(write_permission,0, [write_role, 'Manager', 'Owner'])

	workflowDef.states.addState('Read and write to authenticated')
	wfStrwau = workflowDef.states['Read and write to authenticated']
	wfStrwau.setProperties(title="""Read and write to authenticated""",
				transitions=('Read and write restricted Transition', 'Read to authenticated, write restricted Transition', 'Read to all, write restricted Transition', 'Read to all write to authenticated Transition', 'Read and write to all Transition'))
	wfStrwau.setPermission(add_permission, 0, ['Manager', 'Owner'])
	wfStrwau.setPermission(edit_permission, 0, ['Manager', 'Owner'])
	wfStrwau.setPermission(view_permission, 0, ['Authenticated', 'Manager', 'Owner'])
	wfStrwau.setPermission(webdav_permission, 0, ['Authenticated', 'Manager', 'Owner'])
	wfStrwau.setPermission(write_permission,0, ['Authenticated', 'Manager', 'Owner'])

	workflowDef.states.addState('Read to all, write restricted')
	wfStrallwrest = workflowDef.states['Read to all, write restricted']
	wfStrallwrest.setProperties(title="""Read to all, write restricted""",
				transitions=('Read and write restricted Transition', 'Read to authenticated, write restricted Transition', 'Read and write to authenticated Transition', 'Read to all write to authenticated Transition', 'Read and write to all Transition'))
	wfStrallwrest.setPermission(add_permission, 0, ['Manager', 'Owner'])
	wfStrallwrest.setPermission(edit_permission, 0, ['Manager', 'Owner'])
	wfStrallwrest.setPermission(view_permission, 0, ['Anonymous', 'Manager', 'Owner'])
	wfStrallwrest.setPermission(webdav_permission, 0, [webdav_role, 'Manager', 'Owner'])
	wfStrallwrest.setPermission(write_permission,0, [write_role, 'Manager', 'Owner'])

	workflowDef.states.addState('Read to all write to authenticated')
	wfStrallwau = workflowDef.states['Read to all write to authenticated']
	wfStrallwau.setProperties(title="""Read to all write to authenticated""",
				transitions=('Read and write restricted Transition', 'Read to authenticated, write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 'Read and write to all Transition'))
	wfStrallwau.setPermission(add_permission, 0, ['Manager', 'Owner'])
	wfStrallwau.setPermission(edit_permission, 0, ['Manager', 'Owner'])
	wfStrallwau.setPermission(view_permission, 0, ['Anonymous', 'Manager', 'Owner'])
	wfStrallwau.setPermission(webdav_permission, 0, ['Authenticated', 'Manager', 'Owner'])
	wfStrallwau.setPermission(write_permission,0, ['Authenticated', 'Manager', 'Owner'])

	workflowDef.states.addState('Read and write to all')
	wfStrwall = workflowDef.states['Read and write to all']
	wfStrwall.setProperties(title="""Read and write to all""",
				transitions=('Read and write restricted Transition', 'Read to authenticated, write restricted Transition', 'Read and write to authenticated Transition', 'Read to all, write restricted Transition', 'Read to all write to authenticated Transition'))
	wfStrwall.setPermission(add_permission, 0, ['Manager', 'Owner'])
	wfStrwall.setPermission(edit_permission, 0, ['Manager', 'Owner'])
	wfStrwall.setPermission(view_permission, 0, ['Anonymous', 'Manager', 'Owner'])
	wfStrwall.setPermission(webdav_permission, 0, ['Anonymous', 'Manager', 'Owner'])
	wfStrwall.setPermission(write_permission,0, ['Anonymous', 'Manager', 'Owner'])




	##
	# The different Transitions
	#
	transitionsDef = workflowDef.transitions['Read and write restricted Transition']
	transitionsDef.setProperties(title="""Set read and write restricted""",
						new_state_id="""Read and write restricted""",
						trigger_type=1,
						script_name="""""",
						after_script_name="""""",
						actbox_name="""Read and write restricted""",
						actbox_url="""%(content_url)s/content_rwrestr_form""",
						actbox_category="""workflow""",
						props={'guard_roles': 'Owner'},
						)	

	transitionsDef = workflowDef.transitions['Read to authenticated, write restricted Transition']
	transitionsDef.setProperties(title="""Allow read to authenticated, set write restricted""",
						new_state_id="""Read to authenticated, write restricted""",
						trigger_type=1,
						script_name="""""",
						after_script_name="""""",
						actbox_name="""Read to authenticated, write restricted""",
						actbox_url="""%(content_url)s/content_rauthwrestr_form""",
						actbox_category="""workflow""",
						props={'guard_roles': 'Owner'},
						)	

	transitionsDef = workflowDef.transitions['Read and write to authenticated Transition']
	transitionsDef.setProperties(title="""Allow read and write to authenticated""",
						new_state_id="""Read and write to authenticated""",
						trigger_type=1,
						script_name="""""",
						after_script_name="""""",
						actbox_name="""Read and write to authenticated""",
						actbox_url="""%(content_url)s/content_rwauth_form""",
						actbox_category="""workflow""",
						props={'guard_roles': 'Owner'},
						)	

	transitionsDef = workflowDef.transitions['Read to all, write restricted Transition']
	transitionsDef.setProperties(title="""Allow read to all, set write restricted""",
						new_state_id="""Read to all, write restricted""",
						trigger_type=1,
						script_name="""""",
						after_script_name="""""",
						actbox_name="""Read to all, write restricted""",
						actbox_url="""%(content_url)s/content_rallwrest_form""",
						actbox_category="""workflow""",
						props={'guard_roles': 'Owner'},
						)	

	transitionsDef = workflowDef.transitions['Read to all write to authenticated Transition']
	transitionsDef.setProperties(title="""Allow read to all and write to authenticated""",
						new_state_id="""Read to all write to authenticated""",
						trigger_type=1,
						script_name="""""",
						after_script_name="""""",
						actbox_name="""Read to all write to authenticated""",
						actbox_url="""%(content_url)s/content_rallwauth_form""",
						actbox_category="""workflow""",
						props={'guard_roles': 'Owner'},
						)	
												
	transitionsDef = workflowDef.transitions['Read and write to all Transition']
	transitionsDef.setProperties(title="""Allow read and write to all""",
						new_state_id="""Read and write to all""",
						trigger_type=1,
						script_name="""""",
						after_script_name="""""",
						actbox_name="""Read and write to all""",
						actbox_url="""%(content_url)s/content_rwtoall_form""",
						actbox_category="""workflow""",
						props={'guard_roles': 'Owner'},
						)	

		
	workflowDef.initial_state = 'Read and write restricted'


	# State Variable
	workflowDef.variables.setStateVar('review_state')
	
	##
	# Workflow Variables initialization
	#
	variablesDef = workflowDef.variables['action']
	variablesDef.setProperties(description="""The last transition""",
						default_value="""""",
						default_expr="""transition/getId|nothing""",
						for_catalog=0,
						for_status=1,
						update_always=1,
						props=None)
	
	variablesDef = workflowDef.variables['time']
	variablesDef.setProperties(description="""Time of the last transition""",
						default_value="""""",
						default_expr="""state_change/getDateTime""",
						for_catalog=0,
						for_status=1,
						update_always=1,
						props=None)
	
	variablesDef = workflowDef.variables['comments']
	variablesDef.setProperties(description="""Comments about the last transition""",
						default_value="""""",
						default_expr="""python:state_change.kwargs.get('comment', '')""",
						for_catalog=0,
						for_status=1,
						update_always=1,
						props=None)
	
	variablesDef = workflowDef.variables['actor']
	variablesDef.setProperties(description="""The ID of the user who performed the last transition""",
						default_value="""""",
						default_expr="""user/getId""",
						for_catalog=0,
						for_status=1,
						update_always=1,
						props=None)
	
	variablesDef = workflowDef.variables['review_history']
	variablesDef.setProperties(description="""Provides access to workflow history""",
						default_value="""""",
						default_expr="""state_change/getHistory""",
						for_catalog=0,
						for_status=0,
						update_always=0,
						props={'guard_permissions': 'Request review; Review portal content'})

	return workflowDef
